@@ -15,19 +15,12 @@ import re
 import inspect
 import gzip
 from io import BytesIO
-try:
-    import importlib.resources as pkg_resources
-except ImportError:
-    # Try backported to PY<37 `importlib_resources`.
-    import importlib_resources as pkg_resources
+from typing import Dict, List
+import importlib.resources as pkg_resources
 import importlib
+from urllib.request import urlopen, Request
 from cfnlint.decode.node import dict_node, list_node, str_node
 from cfnlint.data import CloudSpecs
-try:
-    from urllib.request import urlopen, Request
-except ImportError:
-    from urllib2 import urlopen
-
 
 
 LOGGER = logging.getLogger(__name__)
@@ -52,6 +45,7 @@ SPEC_REGIONS = {
     'eu-west-2': 'https://d1742qcu2c1ncx.cloudfront.net/latest/gzip/CloudFormationResourceSpecification.json',
     'eu-west-3': 'https://d2d0mfegowb3wk.cloudfront.net/latest/gzip/CloudFormationResourceSpecification.json',
     'me-south-1': 'https://cfn-resource-specifications-me-south-1-prod.s3.me-south-1.amazonaws.com/latest/gzip/CloudFormationResourceSpecification.json',
+    'me-central-1': 'https://cfn-resource-specifications-me-central-1-prod.s3.me-central-1.amazonaws.com/latest/gzip/CloudFormationResourceSpecification.json',
     'sa-east-1': 'https://d3c9jyj3w509b0.cloudfront.net/latest/gzip/CloudFormationResourceSpecification.json',
     'us-east-1': 'https://d1uauaxba7bl26.cloudfront.net/latest/gzip/CloudFormationResourceSpecification.json',
     'us-east-2': 'https://dnwj8swjjbsbt.cloudfront.net/latest/gzip/CloudFormationResourceSpecification.json',
@@ -68,20 +62,39 @@ REGIONS = list(SPEC_REGIONS.keys())
 
 REGEX_ALPHANUMERIC = re.compile('^[a-zA-Z0-9]*$')
 REGEX_CIDR = re.compile(
-    r'^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(\/([0-9]|[1-2][0-9]|3[0-2]))$')
+    r'^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(\/([0-9]|[1-2][0-9]|3[0-2]))$'
+)
 REGEX_IPV4 = re.compile(
-    r'^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)(\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)){3}$')
+    r'^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)(\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)){3}$'
+)
 REGEX_IPV6 = re.compile(
-    r'^(((?=.*(::))(?!.*\3.+\3))\3?|[\dA-F]{1,4}:)([\dA-F]{1,4}(\3|:\b)|\2){5}(([\dA-F]{1,4}(\3|:\b|$)|\2){2}|(((2[0-4]|1\d|[1-9])?\d|25[0-5])\.?\b){4})\Z', re.I | re.S)
+    r'^(((?=.*(::))(?!.*\3.+\3))\3?|[\dA-F]{1,4}:)([\dA-F]{1,4}(\3|:\b)|\2){5}(([\dA-F]{1,4}(\3|:\b|$)|\2){2}|(((2[0-4]|1\d|[1-9])?\d|25[0-5])\.?\b){4})\Z',
+    re.I | re.S,
+)
 REGEX_DYN_REF = re.compile(r'^.*{{resolve:.+}}.*$')
 REGEX_DYN_REF_SSM = re.compile(r'^.*{{resolve:ssm:[a-zA-Z0-9_\.\-/]+:\d+}}.*$')
-REGEX_DYN_REF_SSM_SECURE = re.compile(r'^.*{{resolve:ssm-secure:[a-zA-Z0-9_\.\-/]+:\d+}}.*$')
+REGEX_DYN_REF_SSM_SECURE = re.compile(
+    r'^.*{{resolve:ssm-secure:[a-zA-Z0-9_\.\-/]+:\d+}}.*$'
+)
 
 
 FUNCTIONS = [
-    'Fn::Base64', 'Fn::GetAtt', 'Fn::GetAZs', 'Fn::ImportValue',
-    'Fn::Join', 'Fn::Split', 'Fn::FindInMap', 'Fn::Select', 'Ref',
-    'Fn::If', 'Fn::Contains', 'Fn::Sub', 'Fn::Cidr']
+    'Fn::Base64',
+    'Fn::GetAtt',
+    'Fn::GetAZs',
+    'Fn::ImportValue',
+    'Fn::Join',
+    'Fn::Split',
+    'Fn::FindInMap',
+    'Fn::Select',
+    'Ref',
+    'Fn::If',
+    'Fn::Contains',
+    'Fn::Sub',
+    'Fn::Cidr',
+    'Fn::Length',
+    'Fn::ToJsonString',
+]
 
 FUNCTIONS_MULTIPLE = ['Fn::GetAZs', 'Fn::Split']
 
@@ -102,33 +115,23 @@ PSEUDOPARAMS = [
     'AWS::Region',
     'AWS::StackId',
     'AWS::StackName',
-    'AWS::URLSuffix'
+    'AWS::URLSuffix',
 ]
 
 LIMITS = {
-    'Mappings': {
-        'number': 200,
-        'attributes': 200,
-        'name': 255  # in characters
-    },
+    'Mappings': {'number': 200, 'attributes': 200, 'name': 255},  # in characters
     'Outputs': {
         'number': 200,
         'name': 255,  # in characters
-        'description': 1024  # in bytes
+        'description': 1024,  # in bytes
     },
     'Parameters': {
         'number': 200,
         'name': 255,  # in characters
-        'value': 4096  # in bytes
-    },
-    'Resources': {
-        'number': 500,
-        'name': 255  # in characters
-    },
-    'template': {
-        'body': 1000000,  # in bytes
-        'description': 1024  # in bytes
+        'value': 4096,  # in bytes
     },
+    'Resources': {'number': 500, 'name': 255},  # in characters
+    'template': {'body': 1000000, 'description': 1024},  # in bytes  # in bytes
     'threshold': 0.9,  # for rules about approaching the other limit values
 }
 
@@ -139,7 +142,7 @@ valid_snapshot_types = [
     'AWS::Neptune::DBCluster',
     'AWS::RDS::DBCluster',
     'AWS::RDS::DBInstance',
-    'AWS::Redshift::Cluster'
+    'AWS::Redshift::Cluster',
 ]
 
 VALID_PARAMETER_TYPES_SINGLE = [
@@ -200,11 +203,11 @@ VALID_PARAMETER_TYPES_LIST = [
 
 VALID_PARAMETER_TYPES = VALID_PARAMETER_TYPES_SINGLE + VALID_PARAMETER_TYPES_LIST
 
+# pylint: disable=missing-class-docstring
 class RegexDict(dict):
-
     def __getitem__(self, item):
         possible_items = {}
-        for k, v  in self.items():
+        for k, v in self.items():
             if isinstance(v, dict):
                 if v.get('Type') == 'MODULE':
                     if re.match(k, item):
@@ -238,6 +241,7 @@ class RegexDict(dict):
         except KeyError:
             return default
 
+
 def get_metadata_filename(url):
     """Returns the filename for a metadata file associated with a remote resource"""
     caching_dir = os.path.join(os.path.dirname(__file__), 'data', 'DownloadsMetadata')
@@ -246,6 +250,7 @@ def get_metadata_filename(url):
 
     return metadata_filename
 
+
 def url_has_newer_version(url):
     """Checks to see if a newer version of the resource at the URL is available
     Always returns true if using Python2.7 due to lack of HEAD request support,
@@ -267,42 +272,49 @@ def url_has_newer_version(url):
     try:
         # Make an initial HEAD request
         req = Request(url, method='HEAD')
-        res = urlopen(req)
+        with urlopen(req) as res:
+            # If we have an ETag value stored and it matches the returned one,
+            # then we already have a copy of the most recent version of the
+            # resource, so don't bother fetching it again
+            if (
+                cached_etag
+                and res.info().get('ETag')
+                and cached_etag == res.info().get('ETag')
+            ):
+                LOGGER.debug(
+                    'We already have a cached version of url %s with ETag value of %s',
+                    url,
+                    cached_etag,
+                )
+                return False
 
     except NameError:
         # We should force an update
         return True
 
-    # If we have an ETag value stored and it matches the returned one,
-    # then we already have a copy of the most recent version of the
-    # resource, so don't bother fetching it again
-    if cached_etag and res.info().get('ETag') and cached_etag == res.info().get('ETag'):
-        LOGGER.debug('We already have a cached version of url %s with ETag value of %s', url, cached_etag)
-        return False
-
     # The ETag value of the remote resource does not match the local one, so a newer version is available
     return True
 
+
 def get_url_content(url, caching=False):
     """Get the contents of a spec file"""
 
-    res = urlopen(url)
+    with urlopen(url) as res:
+        if caching and res.info().get('ETag'):
+            metadata_filename = get_metadata_filename(url)
+            # Load in all existing values
+            metadata = load_metadata(metadata_filename)
+            metadata['etag'] = res.info().get('ETag')
+            metadata['url'] = url  # To make it obvious which url the Tag relates to
+            save_metadata(metadata, metadata_filename)
 
-    if caching and res.info().get('ETag'):
-        metadata_filename = get_metadata_filename(url)
-        # Load in all existing values
-        metadata = load_metadata(metadata_filename)
-        metadata['etag'] = res.info().get('ETag')
-        metadata['url'] = url # To make it obvious which url the Tag relates to
-        save_metadata(metadata, metadata_filename)
-
-    # Continue to handle the file download normally
-    if res.info().get('Content-Encoding') == 'gzip':
-        buf = BytesIO(res.read())
-        f = gzip.GzipFile(fileobj=buf)
-        content = f.read().decode('utf-8')
-    else:
-        content = res.read().decode('utf-8')
+        # Continue to handle the file download normally
+        if res.info().get('Content-Encoding') == 'gzip':
+            buf = BytesIO(res.read())
+            f = gzip.GzipFile(fileobj=buf)
+            content = f.read().decode('utf-8')
+        else:
+            content = res.read().decode('utf-8')
 
     return content
 
@@ -311,7 +323,7 @@ def load_metadata(filename):
     """Get the contents of the download metadata file"""
     metadata = {}
     if os.path.exists(filename):
-        with open(filename, 'r') as metadata_file:
+        with open(filename, 'r', encoding='utf-8') as metadata_file:
             metadata = json.load(metadata_file)
     return metadata
 
@@ -322,23 +334,30 @@ def save_metadata(metadata, filename):
     if not os.path.exists(dirname):
         os.mkdir(dirname)
 
-    with open(filename, 'w') as metadata_file:
+    with open(filename, 'w', encoding='utf-8') as metadata_file:
         json.dump(metadata, metadata_file)
 
 
 def load_resource(package, filename='us-east-1.json'):
     """Load CloudSpec resources
-        :param filename: filename to load
-        :return: Json output of the resource laoded
+    :param filename: filename to load
+    :return: Json output of the resource laoded
     """
+    if sys.version_info >= (3, 9):
+        return json.loads(
+            pkg_resources.files(package)  # pylint: disable=no-member
+            .joinpath(filename)
+            .read_text(encoding='utf-8')
+        )
     return json.loads(pkg_resources.read_text(package, filename, encoding='utf-8'))
 
 
-RESOURCE_SPECS = {}
-REGISTRY_SCHEMAS = []
+RESOURCE_SPECS: Dict[str, dict] = {}
+REGISTRY_SCHEMAS: List[dict] = []
+
 
 def merge_spec(source, destination):
-    """ Recursive merge spec dict """
+    """Recursive merge spec dict"""
 
     for key, value in source.items():
         if isinstance(value, dict):
@@ -351,7 +370,7 @@ def merge_spec(source, destination):
 
 
 def set_specs(override_spec_data):
-    """ Override Resource Specs """
+    """Override Resource Specs"""
 
     excludes = []
     includes = []
@@ -377,7 +396,9 @@ def set_specs(override_spec_data):
         if includes:
             for include in includes:
                 regex = re.compile(include.replace('*', '(.*)') + '$')
-                matches = [string for string in all_resources if re.match(regex, string)]
+                matches = [
+                    string for string in all_resources if re.match(regex, string)
+                ]
 
                 resources.extend(matches)
         else:
@@ -399,12 +420,15 @@ def set_specs(override_spec_data):
 
 
 def is_custom_resource(resource_type):
-    """ Return True if resource_type is a custom resource """
-    return resource_type and (resource_type == 'AWS::CloudFormation::CustomResource' or resource_type.startswith('Custom::'))
+    """Return True if resource_type is a custom resource"""
+    return resource_type and (
+        resource_type == 'AWS::CloudFormation::CustomResource'
+        or resource_type.startswith('Custom::')
+    )
 
 
 def bool_compare(first, second):
-    """ Compare strings to boolean values """
+    """Compare strings to boolean values"""
 
     if isinstance(first, str):
         first = bool(first.lower() in ['true', 'True'])
@@ -416,21 +440,25 @@ def bool_compare(first, second):
 
 
 def initialize_specs():
-    """ Reload Resource Specs """
+    """Reload Resource Specs"""
     for reg in REGIONS:
-        RESOURCE_SPECS[reg] = load_resource(CloudSpecs, filename=('%s.json' % reg))
+        RESOURCE_SPECS[reg] = load_resource(CloudSpecs, filename=(f'{reg}.json'))
 
 
 initialize_specs()
 
 
 def format_json_string(json_string):
-    """ Format the given JSON string"""
+    """Format the given JSON string"""
+
     def converter(o):  # pylint: disable=R1710
-        """ Help convert date/time into strings """
+        """Help convert date/time into strings"""
         if isinstance(o, datetime.datetime):
-            return o.__str__()
-    return json.dumps(json_string, indent=2, sort_keys=True, separators=(',', ': '), default=converter)
+            return o.__str__()  # pylint: disable=unnecessary-dunder-call
+
+    return json.dumps(
+        json_string, indent=1, sort_keys=True, separators=(',', ': '), default=converter
+    )
 
 
 def create_rules(mod):
@@ -438,26 +466,35 @@ def create_rules(mod):
     from the given module."""
     result = []
     for _, clazz in inspect.getmembers(mod, inspect.isclass):
-        if clazz.__name__ == 'CustomRule' and clazz.__module__ == 'cfnlint.rules.custom':
+        if (
+            clazz.__name__ == 'CustomRule'
+            and clazz.__module__ == 'cfnlint.rules.custom'
+        ):
             continue
         method_resolution = inspect.getmro(clazz)
-        if [clz for clz in method_resolution[1:] if clz.__module__ in ('cfnlint', 'cfnlint.rules') and clz.__name__ == 'CloudFormationLintRule']:
+        if [
+            clz
+            for clz in method_resolution[1:]
+            if clz.__module__ in ('cfnlint', 'cfnlint.rules')
+            and clz.__name__ == 'CloudFormationLintRule'
+        ]:
             # create and instance of subclasses of CloudFormationLintRule
             obj = clazz()
             result.append(obj)
     return result
 
 
-
 loader_details = (
     importlib.machinery.SourceFileLoader,  # pylint: disable=no-member
-    importlib.machinery.SOURCE_SUFFIXES  # pylint: disable=no-member
+    importlib.machinery.SOURCE_SUFFIXES,  # pylint: disable=no-member
 )
 
+
 def import_filename(pluginname, root):
-    """ import_filename imports a module from a file"""
+    """import_filename imports a module from a file"""
     mod_finder = importlib.machinery.FileFinder(  # pylint: disable=no-member
-        root, loader_details)
+        root, loader_details
+    )
 
     mod_spec = mod_finder.find_spec(pluginname)
     if mod_spec is not None:
@@ -513,7 +550,7 @@ def override_specs(override_spec_file):
     """Override specs file"""
     try:
         filename = override_spec_file
-        with open(filename) as fp:
+        with open(filename, encoding='utf-8') as fp:
             custom_spec_data = json.load(fp)
 
         set_specs(custom_spec_data)
@@ -522,10 +559,14 @@ def override_specs(override_spec_file):
             LOGGER.error('Override spec file not found: %s', filename)
             sys.exit(1)
         elif e.errno == 21:
-            LOGGER.error('Override spec file references a directory, not a file: %s', filename)
+            LOGGER.error(
+                'Override spec file references a directory, not a file: %s', filename
+            )
             sys.exit(1)
         elif e.errno == 13:
-            LOGGER.error('Permission denied when accessing override spec file: %s', filename)
+            LOGGER.error(
+                'Permission denied when accessing override spec file: %s', filename
+            )
             sys.exit(1)
     except (ValueError) as err:
         LOGGER.error('Override spec file %s is malformed: %s', filename, err)
